package com.persagy.ztkencryptdecodedata.netty;

import com.alibaba.fastjson.JSONObject;
import com.persagy.ztkencryptdecodedata.dataSafety.DecryptInputMessageService;
import com.persagy.ztkencryptdecodedata.dataSafety.EncryptInputMessageService;
import com.persagy.ztkencryptdecodedata.entity.ChattingUser;
import com.persagy.ztkencryptdecodedata.kafka.EdgeKafkaProducer;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.buffer.ByteBuf;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.util.CharsetUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import javax.annotation.PostConstruct;
import java.net.InetSocketAddress;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.TimeUnit;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Component
public class NettyServer {

    private Logger logger = LoggerFactory.getLogger(this.getClass());
    //private static BlockingQueue messageQueue = null;
    BlockingQueue<String> messageQueue = new LinkedBlockingQueue<>(1024 * 1024);
    private SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss:SSS");//格式化

    private  int m=0;

    @Autowired
    private EncryptInputMessageService encryptInputMessageService;

    @Autowired
    private DecryptInputMessageService decryptInputMessageService;

    @Autowired
    private EdgeKafkaProducer edgeKafkaProducer;


    @Value("${listener.port}")
    private String listenerPort;

    private ChannelFuture [] ChannelFutures = null;
    private List<ChattingUser> list = new ArrayList<>();

    public String GetRemoteIP(String str){
        String result ="";
        String regEx="((2[0-4]\\d|25[0-5]|[01]?\\d\\d?)\\.){3}(2[0-4]\\d|25[0-5]|[01]?\\d\\d?)";
        Pattern p = Pattern.compile(regEx);
        Matcher m = p.matcher(str);
        while (m.find()) {
            result = m.group();
           // System.out.println(result);
            //break;   加break则提取string中的一个IP
        }
        return result;
    }




    public static String convertHexToString(String hex) {
        StringBuilder sb = new StringBuilder();
        StringBuilder sb2 = new StringBuilder();
        for (int i = 0; i < hex.length() - 1; i += 2) {
            String s = hex.substring(i, (i + 2));
            int decimal = Integer.parseInt(s, 16);
            sb.append((char) decimal);
            sb2.append(decimal);
        }
        return sb.toString();
    }
    @PostConstruct
    private void start() {

        EventLoopGroup bossGroup = new NioEventLoopGroup();
        NioEventLoopGroup workerGroup = new NioEventLoopGroup();
        ServerBootstrap serverBootstrap = new ServerBootstrap();

        serverBootstrap.group(bossGroup,workerGroup);
        serverBootstrap.channel(NioServerSocketChannel.class);
        serverBootstrap.childOption(ChannelOption.SO_REUSEADDR,true);

        serverBootstrap.childHandler(new ChannelInitializer() {
            @Override
            protected void initChannel(Channel ch) throws Exception {
                ch.pipeline().addLast(new CountHandler());
            }
        });

        if (ChannelFutures == null){
            ChannelFutures = new ChannelFuture[8];
        }



        //多端口绑定
        String[] ipPortArr =listenerPort.split(",");
        for (int i = 1;i <= ipPortArr.length ;i++){
            ChattingUser chattingUser = new ChattingUser();
            chattingUser.setUserId(ipPortArr[i-1].split(":")[1]);
            chattingUser.setIp(ipPortArr[i-1].split(":")[2]);
            int port = Integer.valueOf(ipPortArr[i-1].split(":")[3]);
            chattingUser.setPort(port);
            list.add(chattingUser);

            ChannelFuture channelFuture = serverBootstrap.bind(port);
            ChannelFutures[i] = channelFuture;
            channelFuture.addListener(future -> {
                ;
                if (future.isSuccess()) {
                    System.out.println("Started success,port:" + port+">>>"+ channelFuture.channel().pipeline().channel().id().asLongText());
                } else {
                    System.out.println("Started Failed,port:" + port +">>>"+ channelFuture.channel().pipeline().channel().id().asLongText());
                }
            });

        }
    }



    @ChannelHandler.Sharable
    class CountHandler extends ChannelInboundHandlerAdapter {

//        private AtomicInteger nConnection = new AtomicInteger();
//
//        public CountHandler() {
//            Executors.newSingleThreadScheduledExecutor().scheduleAtFixedRate(() -> {
//                System.out.println("connections: " + nConnection.get());
//            }, 0, 2, TimeUnit.SECONDS);
//
//        }

        @Override
        public void channelActive(ChannelHandlerContext ctx) throws Exception {
            //nConnection.incrementAndGet();
           // System.out.println(">>>>>"+nConnection.incrementAndGet());
            InetSocketAddress inteSocket = (InetSocketAddress) ctx.channel().remoteAddress();

            System.out.println("端口 已有客户端连接***"+inteSocket.getPort());

            super.channelActive(ctx);
        }



        @Override
        public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {
            //super.channelRead(ctx, msg);
           // nConnection.decrementAndGet();
            InetSocketAddress inteSocket = (InetSocketAddress) ctx.channel().localAddress();
            String localip = inteSocket.getAddress().getHostAddress();
            String localport = String.valueOf(inteSocket.getPort());
            System.out.println(
                    "server ip: " + localip + "Server port "+ localport
            );

           // System.out.println(">>>>>"+nConnection.decrementAndGet());
            String time = sdf.format(new Date());
            ByteBuf in = (ByteBuf) msg;
            System.out.println(
                    "server receive: " + in.toString(CharsetUtil.UTF_8)
            );
            String str = in.toString(CharsetUtil.UTF_8);
            JSONObject obj =new JSONObject();
            obj.put("port",localport);
            obj.put("msg",str);
            obj.put("time",time);

             //EncryptInputMessage.beforeBodyWrite(obj);
            for (int j=0;j<list.size();j++){
                if (Integer.valueOf(localport) == list.get(j).getPort() ){
                    obj.put("userid",list.get(j).getUserId());
                    obj.put("targetAddress",list.get(j).getIp());

                }
            }

            messageQueue.offer(obj.toString(), 1000, TimeUnit.MICROSECONDS);

            //System.out.println("dataEncrypt>>>"+encryptInputMessageService.beforeBodyWrite(obj).toString());
           // decryptInputMessageService.DecryptMsgInputMessage(encryptInputMessageService.beforeBodyWrite(obj).toString());
            //System.out.println("dataDecrypt>>>"+ decryptInputMessageService.DecryptMsgInputMessage(encryptInputMessageService.beforeBodyWrite(obj).toString()));

//            if (!messageQueue.isEmpty()&&m==0 ){
//                m++;
//                edgeKafkaProducer.setQueue(messageQueue);
//            }
            edgeKafkaProducer.setQueue(messageQueue);
            /**
             *
             * ctx.write(in);
             * Bytebuf并不适合直接重用，必须新建才能写回客户端，
             * 否则将会报io.netty.util.IllegalReferenceCountException: refCnt: 0, decrement: 1
             * 原因是这是因为Netty4有引用计数器的原因，自从Netty 4开始，对象的生命周期由它们的引用计数（reference counts）管理，
             * 而不是由垃圾收集器（garbage collector）管理了。ByteBuf是最值得注意的，它使用了引用计数来改进分配内存和释放内存的性能。
             * 在我们创建ByteBuf对象后，它的引用计数是1，当你释放（release）引用计数对象时，它的引用计数减1，
             * 如果引用计数为0，这个引用计数对象会被释放（deallocate）,并返回对象池。
             * 当尝试访问引用计数为0的引用计数对象会抛出IllegalReferenceCountException异常：
             * 或者在该Bytebuf 在复用前需要调用retain(),将计数器置为1
             */
           // ByteBuf pong = Unpooled.copiedBuffer(str.getBytes());
            //ctx.write(pong);

           System.out.println(GetRemoteIP(String.valueOf(ctx.channel().remoteAddress()))+"接受到..Server received: " + in.toString(CharsetUtil.UTF_8));
           Channel channel =ctx.channel();
          ;
            //System.out.println( channel.pipeline().get("id"));

           String ip = GetRemoteIP(String.valueOf(ctx.channel().remoteAddress()));
            if (list.contains(ip)) {
                System.out.println("1111");
            }else {
                System.out.println("2222>>>>"+ channel.id().asLongText());
            }

           // ctx.write(in);
            //ctx.write(msg);
           // ctx.flush();

        }
        @Override
        public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
            cause.printStackTrace();
            ctx.close();
        }

    }
}
